pub struct EaseOffCore { /* private fields */ }Expand description
Immutable core backoff API, without error management or sleeps.
Implementations
Sourceimpl EaseOffCore
impl EaseOffCore
Sourcepub const fn new(options: Options) -> Self
pub const fn new(options: Options) -> Self
Create an instance from a built Options.
May be more conveniently invoked as Options::into_core().
Sourcepub fn nth_retry_at(
&self,
n: u32,
now: Instant,
deadline: Option<Instant>,
rng: &mut (impl Rng + ?Sized),
) -> Result<Option<Instant>, RetryAfterDeadline>
pub fn nth_retry_at( &self, n: u32, now: Instant, deadline: Option<Instant>, rng: &mut (impl Rng + ?Sized), ) -> Result<Option<Instant>, RetryAfterDeadline>
Returns the recommended Instant at which to schedule the nth backoff attempt.
Returns Ok(None) if n == 0 and Options::initial_jitter is not greater than zero.
Returns Err if the calculated Instant falls after deadline.