Error

Enum Error 

pub enum Error<E> {
    MaybeRetryable(E),
    Fatal(E),
    TimedOut(TimeoutError<E>),
}
Error type for EaseOff which includes the fatality level of the error.

Variants



MaybeRetryable(E)

The inner error has not been determined to be fatal yet.

RetryableError::can_retry() passes through to the inner error.



Fatal(E)

The error was determined to be fatal.

Always returns false from RetryableError::can_retry().



TimedOut(TimeoutError<E>)

The deadline has elapsed.

Contained is the error from the most recent attempt.

Always returns false from RetryableError::can_retry().

Implementations

Source

impl<E> Error<E>

pub fn on_timeout( self, on_timeout: impl FnOnce(TimeoutError<E>) -> Self, ) -> Self

Convert Error::TimedOut, if applicable, to another variant.

pub fn map<E2>(self, map: impl FnOnce(E) -> E2) -> Error<E2>

Map the inner error type, retaining its retryability status.

pub fn inner(&self) -> &E

Get the inner error.

pub fn into_inner(self) -> E

Unwrap the inner error.