TryAsyncFuture

Struct TryAsyncFuture 

pub struct TryAsyncFuture<'a, E, F, Fut> { /* private fields */ }
Available on crate features async-io-2 or tokio only.
Future returned by TryAsync::into_future(), TryAsync::enforce_deadline_with().

If the current state of the EaseOff prescribes a sleep before the next attempt, the future will not be invoked immediately.

Panics

If an async runtime is not available for sleeping between retries.